#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>    
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>	
	public class VolumeGradientBars : Indicator
	{
		private Brush highVolume = Brushes.Lime;
		private Brush lowVolume = Brushes.Red;
		
		private SMA sma;
		private StdDev std;
		
		private Dictionary<double,Brush> brushes = new Dictionary<double,Brush>();
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Colors bars with a gradient based on volume over a user specified lookback period";
				Name										= "VolumeGradientBars";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				LookBack					= 100;
				HighVolume					= Brushes.Lime;
				LowVolume					= Brushes.Red;
			}
			else if (State == State.Configure)
			{
				Color lowVol = (Color) lowVolume.GetValue(SolidColorBrush.ColorProperty);
				Color highVol = (Color) highVolume.GetValue(SolidColorBrush.ColorProperty);			
				
				for (int x=0;x<=100;x++)
				{
					double pct = x==0 ? 0 : (double)x/100;
					pct = Math.Round(pct,2);
					Color clr = BlendColors(highVol,lowVol,pct);
					Brush brush = new SolidColorBrush(clr);
					brush.Freeze();
					brushes.Add(pct,brush);
				}
			}
			else if (State == State.DataLoaded)
			{
				sma = SMA(Volume,LookBack);
				std = StdDev(Volume,LookBack);
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar<LookBack)
				return;
						
			double devs = (Volume[0]-sma[0])/std[0];
			devs = Math.Min(3,Math.Max(-2,devs));
			devs += 2;
			
			double value = devs==0 ? 0 : devs/5;
			value = Math.Round(value,2);					

			int barNumber = CurrentBar;
			SetBarColor(barNumber, value);			
		}
		
		private void SetBarColor(int barNumber, double value)
		{
			this.Dispatcher.InvokeAsync(() =>
			{								
				if (brushes.ContainsKey(value))				
					BarBrushes.Set(barNumber, brushes[value]);				
			});
		}
		
		private Color BlendColors(Color highVol, Color lowVol, double amount)
		{
			byte r = (byte) ((highVol.R * amount) + lowVol.R * (1 - amount));
			byte g = (byte) ((highVol.G * amount) + lowVol.G * (1 - amount));
			byte b = (byte) ((highVol.B * amount) + lowVol.B * (1 - amount));
			return Color.FromArgb(255, r, g, b);
		}	
		
		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="LookBack", Order=1, GroupName="Parameters")]
		public int LookBack
		{ get; set; }

		[NinjaScriptProperty]
		[XmlIgnore]
		[Display(Name="HighVolume", Order=2, GroupName="Parameters")]
		public Brush HighVolume
		{ 
			get { return highVolume; }
			set
			{
				highVolume = value;
				if (highVolume!=null)
				{
					if (highVolume.IsFrozen)
						highVolume = highVolume.Clone();
					highVolume.Freeze();
				}
			}
		}

		[Browsable(false)]
		public string HighVolumeSerializable
		{
			get { return Serialize.BrushToString(HighVolume); }
			set { HighVolume = Serialize.StringToBrush(value); }
		}			

		[NinjaScriptProperty]
		[XmlIgnore]
		[Display(Name="LowVolume", Order=3, GroupName="Parameters")]
		public Brush LowVolume
		{ 
			get { return lowVolume; }
			set
			{
				lowVolume = value;
				if (lowVolume!=null)
				{
					if (lowVolume.IsFrozen)
						lowVolume = lowVolume.Clone();
					lowVolume.Freeze();
				}
			}
		}

		[Browsable(false)]
		public string LowVolumeSerializable
		{
			get { return Serialize.BrushToString(LowVolume); }
			set { LowVolume = Serialize.StringToBrush(value); }
		}			
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private VolumeGradientBars[] cacheVolumeGradientBars;
		public VolumeGradientBars VolumeGradientBars(int lookBack, Brush highVolume, Brush lowVolume)
		{
			return VolumeGradientBars(Input, lookBack, highVolume, lowVolume);
		}

		public VolumeGradientBars VolumeGradientBars(ISeries<double> input, int lookBack, Brush highVolume, Brush lowVolume)
		{
			if (cacheVolumeGradientBars != null)
				for (int idx = 0; idx < cacheVolumeGradientBars.Length; idx++)
					if (cacheVolumeGradientBars[idx] != null && cacheVolumeGradientBars[idx].LookBack == lookBack && cacheVolumeGradientBars[idx].HighVolume == highVolume && cacheVolumeGradientBars[idx].LowVolume == lowVolume && cacheVolumeGradientBars[idx].EqualsInput(input))
						return cacheVolumeGradientBars[idx];
			return CacheIndicator<VolumeGradientBars>(new VolumeGradientBars(){ LookBack = lookBack, HighVolume = highVolume, LowVolume = lowVolume }, input, ref cacheVolumeGradientBars);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.VolumeGradientBars VolumeGradientBars(int lookBack, Brush highVolume, Brush lowVolume)
		{
			return indicator.VolumeGradientBars(Input, lookBack, highVolume, lowVolume);
		}

		public Indicators.VolumeGradientBars VolumeGradientBars(ISeries<double> input , int lookBack, Brush highVolume, Brush lowVolume)
		{
			return indicator.VolumeGradientBars(input, lookBack, highVolume, lowVolume);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.VolumeGradientBars VolumeGradientBars(int lookBack, Brush highVolume, Brush lowVolume)
		{
			return indicator.VolumeGradientBars(Input, lookBack, highVolume, lowVolume);
		}

		public Indicators.VolumeGradientBars VolumeGradientBars(ISeries<double> input , int lookBack, Brush highVolume, Brush lowVolume)
		{
			return indicator.VolumeGradientBars(input, lookBack, highVolume, lowVolume);
		}
	}
}

#endregion
